local GTT = _G.GameTooltip
local GetAddOnMetadata = 			_G.GetAddOnMetadata
local GetContainerNumSlots = 		_G.GetContainerNumSlots
local GetContainerItemQuestInfo = 	_G.GetContainerItemQuestInfo
local GetContainerItemInfo = 		_G.GetContainerItemInfo
local GetItemInfo = 				_G.GetItemInfo
local GetNumQuestLogEntries = 		_G.GetNumQuestLogEntries
local GetQuestLogTitle = 			_G.GetQuestLogTitle
local GetQuestLogSpecialItemInfo = 	_G.GetQuestLogSpecialItemInfo
local InCombatLockdown = 			_G.InCombatLockdown

local Addon = "QuickQuestItem"
local sides = {"Left","Right"}
local global, character, debug, queuedItem, spaceNeeded

local function Meta(vaL)
	local text = GetAddOnMetadata(Addon, vaL)
	return text
end

local defaults = {
	last = nil,
	itemlist = {},
	useTargets = false,
	targets = "",
	version = tonumber(Meta("Version"))
}

local function copyTable(input)
	local tempTable = {}
	for key, value in pairs(input) do
		if type(value) == "table" then tempTable[key] = copyTable(value)
		else tempTable[key] = value end
	end
	return tempTable
end

QQI_ParsedItems = {}
QQI_QuestItems = {}
QQI_DailyItems = {}

-- To do list:
-- 1. Create a way to detect recently used quest items that do not match the QQI_DB.itemlist.
-- 2. Create a timed delay after targeting or mouse'ingover a unit in which the macro resets to the QQI_ParsedItems table.
-- 3. Use the isDaily parameter to watch for zone changes and detect 'use' items in various zones.
-- 4. Options of course.

local function IsInInventory(itemName)
	for b = 0,4,1 do 
		for s = 1,GetContainerNumSlots(b),1 do 
			local _,_,_,_,readable,_,itemLink = GetContainerItemInfo(b,s)
			if itemLink then
				local itemString = GetItemInfo(itemLink)
				if itemName == itemString then
					return true
				end
			end
		end 
	end
	return false
end

function QQI_ParseInventory()
	QQI_ParsedItems = {} -- Clear the table.
	for b = 0,4,1 do 
		for s = 1,GetContainerNumSlots(b),1 do 
			local IsQuestItem,StartsQuest,_ = GetContainerItemQuestInfo(b,s)
			local _,_,_,_,readable,_,itemLink = GetContainerItemInfo(b,s)
			--local startTime, duration, isEnabled = GetContainerItemCooldown(b,s)
			if IsQuestItem and not StartsQuest and not readable then
				local itemString = GetItemInfo(itemLink)
				if not tContains(QQI_ParsedItems,itemString) then
					tinsert(QQI_ParsedItems,itemString)
				end
			end
		end 
	end
	QQI_UpdateMacro()
end

function QQI_ParseQuestLog()
	QQI_QuestItems = {} -- Clear the table.
	--QQI_DailyItems = {}
	for i=1,GetNumQuestLogEntries() do
		local questLogTitleText,_,_,_,isHeader,_,_,isDaily,_,_ = GetQuestLogTitle(i) -- isDaily may possibly be used later
		if not isHeader then
			local link,icon,charges = GetQuestLogSpecialItemInfo(i) -- charges may play a role later
			--[[if isDaily then
				if link then
					local itemName = GetItemInfo(link)
					QQI_DailyItems[questLogTitleText] = itemName
				else
					QQI_DailyItems[questLogTitleText] = nil
				end
			else]]
				if link then
					local itemName = GetItemInfo(link)
					QQI_QuestItems[questLogTitleText] = itemName
				else
					QQI_QuestItems[questLogTitleText] = nil
				end
			--end
		end
	end
	QQI_UpdateMacro()
end

function QQI_ParseTooltip()
	for i=1,2 do -- sides[i] checks 'Left' and 'Right'
		for n=1,GTT:NumLines() do
			if _G["GameTooltipText"..sides[i]..n] then 
				local TooltipText = _G["GameTooltipText"..sides[i]..n]:GetText()
				if TooltipText then
					for k,v in pairs(QQI_QuestItems) do
						if k == TooltipText and v ~= nil then -- May need to be changed to string.find / match for more accurate results
							if UnitExists("mouseover") then
								QQI_DB.itemlist[UnitName("mouseover")] = v
								QQI_UpdateMacro("mouseover")
							end
						end
					end
				end
			end
		end
	end
end

function QQI_MacroFinder()
	global,character = GetNumMacros()
	local found = false
	for i=1, global do
		local name = GetMacroInfo(i)
		if name == "QQI_MACRO" then
			found = true
			return true, i
		end
	end
	if not found then
		return false, nil
	end
end



function QQI_CreateMacro()
	if spaceNeeded then	return end
	if InCombatLockdown() then return end
	
	global,character = GetNumMacros()
	local isFound, macroSlot = QQI_MacroFinder()
	local hasSpace = global < MAX_ACCOUNT_MACROS -- Changed in 6.0.2
	if hasSpace then 
		if not isFound and not InCombatLockdown() then
			CreateMacro("QQI_MACRO","INV_MISC_QUESTIONMARK","--QQI_MACRO--",nil,nil)
			QQI_ParseQuestLog()
			QQI_ParseInventory()
			QQI_UpdateMacro()
		end
	else
		print(Meta("Title")..": No global macro space. Please delete a macro to create space.")
		spaceNeeded = true
	end
end

function QQI_MacroUpdater(macroSlot,itemName)
	if macroSlot and itemName then
		EditMacro(macroSlot, "QQI_MACRO","INV_MISC_QUESTIONMARK","#showtooltip\n/use "..itemName,nil,nil)
	elseif macroSlot and not itemName then
		EditMacro(macroSlot, "QQI_MACRO","Icon_PetFamily_Magical","--QQI MACRO--\n--No items available!--",nil,nil)
	end
end

function QQI_UpdateMacro(unit)
	local isFound,macroSlot = QQI_MacroFinder()
	if isFound and macroSlot then
		if unit then -- Unit Update is the only point we save QQI_DB.last since we have a Unit we can match a quest item to.
			local itemName = QQI_DB.itemlist[UnitName(unit)]
			if itemName and IsInInventory( itemName ) then
				if not InCombatLockdown() then
					QQI_MacroUpdater( macroSlot, itemName )
				else -- Only places information for QQI_DB.last if we're in combat.
					QQI_DB.last = itemName
					queuedItem = true
				end
			end
		else -- Use Parsed Item List
			if not InCombatLockdown() then
				if queuedItem then
					if QQI_DB.last then
						QQI_MacroUpdater( macroSlot, QQI_DB.last )
						QQI_DB.last = nil
						queuedItem = false
					end
				else
					if QQI_ParsedItems and #QQI_ParsedItems > 0 then
						local useList = ""
						for f=0,#QQI_ParsedItems do -- Creates a multiline string for the macro to use all parsed items. (This is affected by macro character limits of course)
							if f == 1 then
								useList = QQI_ParsedItems[f]
							elseif f > 1 then
								useList = useList.."\n/use "..QQI_ParsedItems[f]
							end
						end
						QQI_MacroUpdater( macroSlot, useList )
					else
						QQI_MacroUpdater( macroSlot, false)
					end
				end				
			end
		end
	else
		QQI_CreateMacro()
	end
end

local QQI = CreateFrame("Frame","QQI")
QQI:SetScript("OnEvent",function(self,event,...)
	local arg1,arg2,arg3,arg4 = ...
	if event == "ADDON_LOADED" and arg1 == Addon then
		if not QQI_DB or not QQI_DB.version or tonumber(Meta("Version")) > QQI_DB.version then
			QQI_DB = copyTable(defaults)
		end
		self:UnregisterEvent("ADDON_LOADED")
	elseif event == "PLAYER_ENTERING_WORLD" then
		QQI_CreateMacro()
	elseif event == "PLAYER_TARGET_CHANGED" then
		QQI_UpdateMacro("target")
	elseif event == "UPDATE_MOUSEOVER_UNIT" then
		QQI_UpdateMacro("mouseover")
	elseif event == "PLAYER_REGEN_ENABLED" then
		QQI_UpdateMacro()
	elseif event == "QUEST_LOG_UPDATE" or (event == "UNIT_QUEST_LOG_CHANGED" and arg1 == "player") then
		QQI_ParseQuestLog()
		self:UnregisterEvent("QUEST_LOG_UPDATE")
	elseif event == "BAG_UPDATE" then
		QQI_ParseInventory()
	elseif event == "PLAYER_ALIVE" then
		GTT:HookScript("OnShow",QQI_ParseTooltip)
	else
		if debug then
			if arg2 then
				print(event.." "..arg1.." "..arg2)
			elseif arg1 then
				print(event.." "..arg1)
			else
				print(event)
			end
		end
	end
end)

QQI:RegisterEvent("ADDON_LOADED")
QQI:RegisterEvent("PLAYER_ALIVE")
QQI:RegisterEvent("PLAYER_FOCUS_CHANGED")
QQI:RegisterEvent("PLAYER_TARGET_CHANGED")
QQI:RegisterEvent("PLAYER_ENTERING_WORLD")
QQI:RegisterEvent("PLAYER_REGEN_ENABLED")
QQI:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
QQI:RegisterEvent("BAG_UPDATE")
QQI:RegisterEvent("QUEST_LOG_UPDATE")
QQI:RegisterEvent("UNIT_QUEST_LOG_CHANGED")